#include "AITank.h"
#include "Defines.h"
#include "D3DXMathFunctions.h"
#include "DirectX9Game.h"
#include "ActorFactory.h"
#include "PlayerTank.h"

AITank::AITank(int ID):Actor(ID), m_Tank(ID),m_Health(100), m_Steering(this)
{
	m_LastBaseYaw = 0.0f;
	m_LastTurretYaw = 0.0f;
	m_StartingPause = 4.0f;
}


void AITank::Update(float dt)
{
	m_Tank.Update(dt);
	if(m_Health <= 0)
	{
		DirectX9Game::Instance()->AddScore(500);
		gMessenger->AddMessage(m_ID, m_ID, msg_Destroy, 0.0f);
	}
	D3DXVECTOR3 directionToPlayer = (((PlayerTank*)gActors->GetPlayer())->GetTank()->GetPhysics()->GetPosition() - m_Tank.GetPhysics()->GetPosition());
	float distanceToPlayer = D3DXVec3Length(&directionToPlayer);

	if(distanceToPlayer < 80.0f)
		m_Steering.Flee(((PlayerTank*)gActors->GetPlayer())->GetTank()->GetPhysics()->GetPosition(), 1.0f);
	else if(distanceToPlayer > 100.0f)
		m_Steering.Seek(((PlayerTank*)gActors->GetPlayer())->GetTank()->GetPhysics()->GetPosition(), 1.0f);
	else
	{
		m_Steering.SeekOff();
		m_Steering.FleeOff();
	}
	D3DXVECTOR3 steeringForce = m_Steering.Calculate().linear;
	steeringForce.y = 0.0f;
	m_Tank.GetPhysics()->Accelerate(steeringForce);

	//Look where you're going.
	float yawRot = atan2(m_Tank.GetPhysics()->GetVelocity().x, m_Tank.GetPhysics()->GetVelocity().z);
	m_Tank.RotateBase(yawRot - m_LastBaseYaw);
	m_LastBaseYaw = yawRot;

	//Track turret to player
	D3DXVec3Normalize(&directionToPlayer, &directionToPlayer);
	yawRot = atan2(directionToPlayer.x , directionToPlayer.z);
	m_Tank.RotateTurret(yawRot - m_LastTurretYaw);
	m_LastTurretYaw = yawRot;

	//Weapons locked for a few seconds when starting up.
	if(m_StartingPause > 0.0f)
		m_StartingPause -= dt;

	if(m_StartingPause <= 0.0f)
	{
		m_Tank.ReloadMachineGuns(50);
		m_Tank.ReloadMainGun(10);
		int mainGunChance = rand()%800;
		int machineGunChance = rand()%200;
		if(machineGunChance == 0)
			m_Tank.FireMachineGuns();
		if(mainGunChance == 0)
			m_Tank.FireMainGun();
	}
}


void AITank::Render()
{
	m_Tank.Render();
}

void AITank::ReceiveMessage(Telegram incoming)
{
	switch(incoming.msg)
	{
	case msg_Destroy:
		if(incoming.sender = m_ID)
			gActors->RemoveActor(m_ID);
		D3DXVECTOR3 newSpawnPoint;
		newSpawnPoint.y = 0.0f;
		newSpawnPoint.x = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
		newSpawnPoint.z = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
		gFactory->CreateEnemyTank(newSpawnPoint);
	}
}

const int AITank::m_MaxHealth = 100;