#include "ActorFactory.h"
#include "D3DXMathFunctions.h"
#include "Actor.h"
#include "Tank.h"
#include "Camera.h"
#include "AITank.h"
#include "PlayerTank.h"
#include "HUD.h"
#include "SkyBox.h"
#include "Projectile.h"
#include "Pickups.h"

ActorFactory* ActorFactory::Instance()
{
	static ActorFactory instance;
	return &instance;
}

int ActorFactory::CreateCamera(D3DXVECTOR3 position, D3DXVECTOR3 lookAt)
{
	D3DXVECTOR3 sideVector;
	D3DXVec3Cross(&sideVector, &lookAt, &CAMERA_UP_VECTOR);
	Camera* camera = new Camera(m_NextValidID, position, CAMERA_UP_VECTOR, sideVector, lookAt, CAMERA_FOV, CAMERA_NEAR_PLANE, CAMERA_FAR_PLANE);
	gActors->SetActiveCamera(camera);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreatePlayer(D3DXVECTOR3 position)
{
	PlayerTank* tank = new PlayerTank(m_NextValidID);
	tank->GetTank()->GetPhysics()->Init(position, D3DXZEROVEC3, TANK_CONTACT_RADIUS, TANK_MAX_ACCEL, TANK_MAX_SPEED, TANK_MAX_ROTATE_SPEED, D3DXVECTOR3(0.0f,-200.0f,0.0f));
	gActors->SetPlayer(tank);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateEnemyTank(D3DXVECTOR3 position)
{
	AITank* tank = new AITank(m_NextValidID);
	tank->GetTank()->GetPhysics()->Init(position, D3DXZEROVEC3, TANK_CONTACT_RADIUS, TANK_MAX_ACCEL, TANK_MAX_SPEED, TANK_MAX_ROTATE_SPEED, D3DXVECTOR3(0.0f,-200.0f,0.0f));
	gActors->SetActiveAITank(tank);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateSkyBox()
{
	gActors->AddActor(new SkyBox(m_NextValidID));
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateHUD()
{
	gActors->AddActor(new HUD(m_NextValidID));
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateShell(D3DXVECTOR3 position, D3DXVECTOR3 direction, int owner)
{
	Projectile* shell = new Projectile(m_NextValidID, owner, SHELL_FLIGHT_TIME, SHELL_DAMAGE, SHELL_SCALE);
	shell->GetPhysics()->Init(position, D3DXZEROVEC3, 0.0f, SHELL_MAX_ACCEL, SHELL_MAX_SPEED, 0.0f, D3DXVECTOR3(0.0f,-10.0f,0.0f));
	shell->GetPhysics()->Accelerate(direction * SHELL_MAX_ACCEL);
	gActors->AddActor(shell);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateBullet(D3DXVECTOR3 position, D3DXVECTOR3 direction, int owner)
{
	Projectile* bullet = new Projectile(m_NextValidID, owner, BULLET_FLIGHT_TIME, BULLET_DAMAGE, BULLET_SCALE);
	bullet->GetPhysics()->Init(position, D3DXZEROVEC3, 0.0f, BULLET_MAX_ACCEL, BULLET_MAX_SPEED, 0.0f, D3DXVECTOR3(0.0f,-10.0f,0.0f));
	bullet->GetPhysics()->Accelerate(direction * BULLET_MAX_ACCEL);
	gActors->AddActor(bullet);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateAmmoPickup(D3DXVECTOR3 position)
{
	AmmoPickup* pickup = new AmmoPickup(m_NextValidID);
	pickup->SetPosition(position);
	gActors->AddActor(pickup);
	m_NextValidID++;
	return m_NextValidID - 1;
}

int ActorFactory::CreateRepairPickup(D3DXVECTOR3 position)
{
	RepairPickup* pickup = new RepairPickup(m_NextValidID);
	pickup->SetPosition(position);
	gActors->AddActor(pickup);
	m_NextValidID++;
	return m_NextValidID - 1;
}