// ActorFactory.h - Provides methods to create applicable actors and add them to the Actor Manager

#pragma once

#include "ActorManager.h"


//Defined starting values:

//Tanks and Cannons
#define STARTING_HEALTH 100
#define STARTING_TANK_SHELLS 5
#define STARTING_TANK_BULLETS 100
#define STARTING_CANNON_SHELLS 10
#define TANK_CONTACT_RADIUS 3.5f
#define TANK_MAX_ACCEL 130
#define TANK_MAX_SPEED 50
#define TANK_MAX_ROTATE_SPEED 2

//Projectiles - Shell
#define SHELL_MAX_ACCEL 200000
#define SHELL_FLIGHT_TIME 5
#define SHELL_MAX_SPEED 1000
#define SHELL_DAMAGE 20
#define SHELL_SCALE 3.0f

//Bullet
#define BULLET_MAX_ACCEL 200000
#define BULLET_FLIGHT_TIME 2
#define BULLET_MAX_SPEED 4000
#define BULLET_DAMAGE 1
#define BULLET_SCALE 0.7f

//Camera
#define CAMERA_FOV D3DXToRadian(40.0f)
#define CAMERA_NEAR_PLANE 1.0f
#define CAMERA_FAR_PLANE 10000.0f
#define CAMERA_UP_VECTOR D3DXVECTOR3(0.0f,1.0f,0.0f)

class ActorFactory
{
	int m_NextValidID;
	ActorFactory():m_NextValidID(0){}
	ActorFactory(ActorFactory&){}
	void operator=(ActorFactory&){}
public:
	static ActorFactory* Instance();

	//Factory Methods
	int CreateCamera(D3DXVECTOR3 position, D3DXVECTOR3 lookAt);
	int CreatePlayer(D3DXVECTOR3 position);
	int CreateEnemyTank(D3DXVECTOR3 position);
	int CreateSkyBox();
	int CreateHUD();
	int CreateShell(D3DXVECTOR3 position, D3DXVECTOR3 direction, int owner);
	int CreateBullet(D3DXVECTOR3 position, D3DXVECTOR3 direction, int owner);
	int CreateAmmoPickup(D3DXVECTOR3 position);
	int CreateRepairPickup(D3DXVECTOR3 position);
};
