#include "ActorManager.h"
#include "Defines.h"
#include "DirectX9App.h"

ActorManager::ActorManager()
{
		ZeroMemory(&m_MainLight, sizeof(m_MainLight));

		m_MainLight.Type = D3DLIGHT_POINT;
		m_MainLight.Position = D3DXVECTOR3(0.0f, 400.0f, 0.0f);
		m_MainLight.Range = 30000.0f;
		m_MainLight.Attenuation0 = 0.0f;
		m_MainLight.Attenuation1 = 0.05f;
		m_MainLight.Attenuation2 = 0.0f;
		m_MainLight.Ambient.a = 0.3f;
		m_MainLight.Ambient.r = 0.3f;
		m_MainLight.Ambient.g = 0.3f;
		m_MainLight.Ambient.b = 0.3f;
		m_MainLight.Specular.a = 1.0f;
		m_MainLight.Specular.r = 1.0f;
		m_MainLight.Specular.g = 1.0f;
		m_MainLight.Specular.b = 1.0f;
		m_MainLight.Diffuse.a = 1.0f;
		m_MainLight.Diffuse.r = 1.0f;
		m_MainLight.Diffuse.g = 1.0f;
		m_MainLight.Diffuse.b = 1.0f;

		HR(gD3DDevice->SetLight(NULL, &m_MainLight));
		HR(gD3DDevice->LightEnable(NULL, true));
}

ActorManager* ActorManager::Instance()
{
	static ActorManager instance;
	return &instance;
}

Actor* ActorManager::GetActor(int ID)
{
	if(m_ActorMap.find(ID) != m_ActorMap.end())
	{
		return m_ActorMap[ID];
	}
	else return nullptr;
}

void ActorManager::AddActor(Actor* Actor)
{
	if(m_ActorMap.find(Actor->GetID()) == m_ActorMap.end())
	{
		m_ActorMap[Actor->GetID()] = Actor;
	}
}

void ActorManager::RemoveActor(int ID)
{
	if(m_ActorMap.find(ID) != m_ActorMap.end())
	{
		delete m_ActorMap[ID];
		m_ActorMap.erase(ID);
	}
}

void ActorManager::Update(float dt)
{
	std::map<int, Actor*>::iterator itr;
	for(itr = m_ActorMap.begin(); itr != m_ActorMap.end(); ++itr)
	{
		itr->second->Update(dt);
	}
}

void ActorManager::Render()
{
	std::map<int, Actor*>::iterator itr;
	for(itr = m_ActorMap.begin(); itr != m_ActorMap.end(); ++itr)
	{
		itr->second->Render();
	}
}

void ActorManager::Reset()
{
	std::map<int, Actor*>::iterator itr;
	for(itr = m_ActorMap.begin(); itr != m_ActorMap.end(); ++ itr)
	{
		delete itr->second;
	}
	m_ActorMap.clear();
}