#include "AssetManager.h"
#include "DirectX9App.h"

AssetManager::AssetManager()
{
	m_MeshMap.clear();
	m_ShaderMap.clear();
	m_TextureMap.clear();
	m_GameLoaded = false;
	m_MenuLoaded = false;
}

AssetManager* AssetManager::Instance()
{
	static AssetManager instance;
	return &instance;
}

AssetManager::~AssetManager()
{
	std::map<std::string, IDirect3DTexture9*>::const_iterator texItr = m_TextureMap.begin();
	std::map<std::string, ID3DXMesh*>::const_iterator meshItr = m_MeshMap.begin();
	std::map<std::string, ID3DXEffect*>::const_iterator shaderItr = m_ShaderMap.begin();
	for(;texItr != m_TextureMap.end(); texItr++)
		texItr->second->Release();
	for(;meshItr != m_MeshMap.end(); meshItr++)
		meshItr->second->Release();
	for(;shaderItr != m_ShaderMap.end(); shaderItr++)
		shaderItr->second->Release();
}

void AssetManager::LoadGameAssets()
{
	if(m_GameLoaded == false)
	{
		IDirect3DTexture9* texIn;
		ID3DXMesh*	meshIn;
		ID3DXEffect* shaderIn;
		ID3DXBuffer* effectError;
		//Primary Shader
		HR(D3DXCreateEffectFromFile(gD3DDevice, "ProjectB.fx", 0, 0, 0, 0, &shaderIn, &effectError))
		m_ShaderMap["ProjectB.fx"] = shaderIn;
	
		//Tank Assets
		HR(D3DXLoadMeshFromX("Meshes/TankBase.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["TankBase.x"] = meshIn;
		HR(D3DXLoadMeshFromX("Meshes/TankTurret.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["TankTurret.x"] = meshIn;
		HR(D3DXLoadMeshFromX("Meshes/TankBarrel.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["TankBarrel.x"] = meshIn;
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/GreyTank.png", &texIn));
		m_TextureMap["GreyTank.png"] = texIn;
	
		//Projectile Assets
		HR(D3DXLoadMeshFromX("Meshes/Bullet.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["Bullet.x"] = meshIn;
	
		//HUD Assets
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/HUD/Tank_HUD.png", &texIn));
		m_TextureMap["Tank_HUD.png"] = texIn;
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/HUD/HUD_Shell.png", &texIn));
		m_TextureMap["HUD_Shell.png"] = texIn;
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/HUD/HUD_Bullet.png", &texIn));
		m_TextureMap["HUD_Bullet.png"] = texIn;
	
		//Skybox Assets
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/SkyBox.png", &texIn));
		m_TextureMap["SkyBox.png"] = texIn;
	
		//Pickup Assets
		HR(D3DXLoadMeshFromX("Meshes/Wrench.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["Wrench.x"] = meshIn;
		HR(D3DXLoadMeshFromX("Meshes/Ammo.x", D3DXMESH_MANAGED, gD3DDevice, NULL, NULL, NULL, NULL, &meshIn));
		m_MeshMap["Ammo.x"] = meshIn;
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/WrenchMetal.png", &texIn));
		m_TextureMap["WrenchMetal.png"] = texIn;
		HR(D3DXCreateTextureFromFile(gD3DDevice, "Images/TankRound.png", &texIn));
		m_TextureMap["TankRound.png"] = texIn;

		m_GameLoaded = true;
	}
}
	
void AssetManager::LoadMenuAssets()
{
	if(m_MenuLoaded == false)
	{
		IDirect3DTexture9* texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/High_Score_Header.png", &texIn);
		m_TextureMap["High_Score_Header.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Exit_Game_Selected.png", &texIn);
		m_TextureMap["Exit_Game_Selected.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Exit_Game.png", &texIn);
		m_TextureMap["Exit_Game.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Player_Guide_Text.png", &texIn);
		m_TextureMap["Player_Guide_Text.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/High_Score.png", &texIn);
		m_TextureMap["High_Score.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/High_Score_Selected.png", &texIn);
		m_TextureMap["High_Score_Selected.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Start_Game_Selected.png", &texIn);
		m_TextureMap["Start_Game_Selected.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Start_Game.png", &texIn);
		m_TextureMap["Start_Game.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Credits.png", &texIn);
		m_TextureMap["Credits.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Credits_Selected.png", &texIn);
		m_TextureMap["Credits_Selected.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Main_Menu_Title.png", &texIn);
		m_TextureMap["Main_Menu_Title.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Credits_Text.png", &texIn);
		m_TextureMap["Credits_Text.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Main_Menu.png", &texIn);
		m_TextureMap["Main_Menu.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Main_Menu_Selected.png", &texIn);
		m_TextureMap["Main_Menu_Selected.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Game_Over_Text.png", &texIn);
		m_TextureMap["Game_Over_Text.png"] = texIn;
		D3DXCreateTextureFromFile(gD3DDevice, "Images/Menu/Tank_Background.png", &texIn);
		m_TextureMap["Tank_Background.png"] = texIn;
	
		m_MenuLoaded = true;
	}
}

IDirect3DTexture9* AssetManager::GetTexture(std::string file)
{
	if(m_TextureMap.find(file) != m_TextureMap.end())
	{
		return m_TextureMap[file];
	}
	else
		return nullptr;
}

ID3DXMesh* AssetManager::GetMesh(std::string file)
{
	if(m_MeshMap.find(file) != m_MeshMap.end())
	{
		return m_MeshMap[file];
	}
	else
		return nullptr;
}

ID3DXEffect* AssetManager::GetShader(std::string file)
{
	if(m_ShaderMap.find(file) != m_ShaderMap.end())
	{
		return m_ShaderMap[file];
	}
	else
		return nullptr;
}

void AssetManager::onResetDevice()
{
	std::map<std::string, ID3DXEffect*>::const_iterator itr;
	for(itr = m_ShaderMap.begin(); itr != m_ShaderMap.end(); itr++)
	{
		itr->second->OnResetDevice();
	}
}

void AssetManager::onLostDevice()
{
	std::map<std::string, ID3DXEffect*>::const_iterator itr;
	for(itr = m_ShaderMap.begin(); itr != m_ShaderMap.end(); itr++)
	{
		itr->second->OnLostDevice();
	}
}