#include "Camera.h"
#include "D3DXMathFunctions.h"
#include "DirectX9App.h"

Camera::Camera(int ID, D3DXVECTOR3 position, D3DXVECTOR3 Y, D3DXVECTOR3 X, D3DXVECTOR3 Z, float FOV, float nearPlane, float farPlane):Actor(ID),
		m_Position(position),
		m_YAxis(Y),
		m_XAxis(X),
		m_ZAxis(Z),
		m_FOV(FOV),
		m_NearPlane(nearPlane),
		m_FarPlane(farPlane)

{


	//Normalize axes
	D3DXVec3Normalize(&m_XAxis, &m_XAxis);
	D3DXVec3Normalize(&m_YAxis, &m_YAxis);
	D3DXVec3Normalize(&m_ZAxis, &m_ZAxis);
	//Generate view matrix
	D3DXMatrixLookAtLH(&m_ViewMatrix, &m_Position, &(m_Position + m_ZAxis), &m_YAxis);
	//Generate projection matrix
	D3DXMatrixPerspectiveFovLH(&m_ProjectionMatrix, m_FOV, gApplication->GetWindowSize().x/gApplication->GetWindowSize().y, m_NearPlane, m_FarPlane);
	//Apply both matrices
	gD3DDevice->SetTransform(D3DTS_VIEW, &m_ViewMatrix);
	gD3DDevice->SetTransform(D3DTS_PROJECTION, &m_ProjectionMatrix);
}

void Camera::Yaw(float amount)
{
	//Rotate X and Z axes about the Y axis
	D3DXMATRIX rotation;
	D3DXMatrixRotationAxis(&rotation, &m_YAxis, amount);
	m_XAxis = TransformVector(&m_XAxis, &rotation);
	m_ZAxis = TransformVector(&m_ZAxis, &rotation);
}

void Camera::Pitch(float amount)
{
	//Rotate Y and Z axes about the X axis
	D3DXMATRIX rotation;
	D3DXMatrixRotationAxis(&rotation, &m_XAxis, amount);
	m_YAxis = TransformVector(&m_YAxis, &rotation);
	m_ZAxis = TransformVector(&m_ZAxis, &rotation);
}

void Camera::Roll(float amount)
{
	//Rotate X and Y axes about the Z axis
	D3DXMATRIX rotation;
	D3DXMatrixRotationAxis(&rotation, &m_ZAxis, amount);
	m_YAxis = TransformVector(&m_YAxis, &rotation);
	m_XAxis = TransformVector(&m_XAxis, &rotation);
}

void Camera::Update(float dt)
{
	//Normalize axes
	D3DXVec3Normalize(&m_XAxis, &m_XAxis);
	D3DXVec3Normalize(&m_YAxis, &m_YAxis);
	D3DXVec3Normalize(&m_ZAxis, &m_ZAxis);

	//Generate view matrix
	D3DXMatrixLookAtLH(&m_ViewMatrix, &m_Position, &(m_Position + m_ZAxis), &m_YAxis);
	//Generate projection matrix
	D3DXMatrixPerspectiveFovLH(&m_ProjectionMatrix, m_FOV, gApplication->GetWindowSize().x/gApplication->GetWindowSize().y, m_NearPlane, m_FarPlane);
	//Apply both matrices
	gD3DDevice->SetTransform(D3DTS_VIEW, &m_ViewMatrix);
	gD3DDevice->SetTransform(D3DTS_PROJECTION, &m_ProjectionMatrix);
}