// Camera.h - 3D Camera for DirectX 9

#pragma once

#include "d3dx9.h"
#include "Actor.h"
#include "Messenger.h"

class Camera : public Actor
{
	D3DXVECTOR3 m_Position;					//Camera's position
	D3DXVECTOR3 m_YAxis;					//Local up vector for camera
	D3DXVECTOR3 m_XAxis;					//Local side vector for camera
	D3DXVECTOR3 m_ZAxis;					//Local forward (look at) vector for camera
	D3DXMATRIX  m_ViewMatrix;				//View Matrix
	D3DXMATRIX  m_ProjectionMatrix;			//Projection Matrix
	float		m_FOV;						//Field of View
	float		m_NearPlane;				//Frustrum Near plane
	float		m_FarPlane;					//Frustrum Far plane

public:
	Camera(int ID, D3DXVECTOR3 position, 
		D3DXVECTOR3 Y, D3DXVECTOR3 X, D3DXVECTOR3 Z, 
		float FOV, float nearPlane, float farPlane
		);
	~Camera(){};

	//Modifiers
	void SetPosition(D3DXVECTOR3 position){m_Position = position;}
	void SetFOV(float FOV){m_FOV = FOV;}
	void SetNearPlane(float nearPlane){m_NearPlane = nearPlane;}
	void SetFarPlane(float farPlane){m_FarPlane = farPlane;}

	//Accessors
	D3DXMATRIX GetView(){return m_ViewMatrix;}
	D3DXMATRIX GetProjection(){return m_ProjectionMatrix;}
	D3DXVECTOR3 GetXAxis(){return m_XAxis;}
	D3DXVECTOR3 GetYAxis(){return m_YAxis;}
	D3DXVECTOR3 GetZAxis(){return m_ZAxis;}
	D3DXVECTOR3 GetPosition(){return m_Position;}
	float GetFOV(){return m_FOV;}
	float GetNearPlane(){return m_NearPlane;}
	float GetFarPlane(){return m_FarPlane;}


	//Rotational Motion
	void Yaw(float amount);
	void Roll(float amount);
	void Pitch(float amount);

	void Update(float dt);

	void Render(){}
	void ReceiveMessage(Telegram){}
};