//////////////////////////////////////////////////////////////////////////////
//	DirectInput.h
//	Tylor Emmett - 2/25/2014
//	Provides Input from Mouse and Keyboard using DirectInput8.
//	Singleton Class.
//	Access via the Instance() method, defined as gInput in Defines.h
//	Init method is called from the DirectX9App WinMain function.
//	Shutdown method is called from the DirectX9App Shutdown method.
///////////////////////////////////////////////////////////////////////////////

#pragma once
#define DIRECTINPUT_VERSION 0x0800
#include <dinput.h>
#include <DxErr.h>
#include <d3dx9.h>
#include <string>

//Cooperation flags
#define KEYBOARD_COOP_FLAGS DISCL_FOREGROUND | DISCL_NONEXCLUSIVE
#define MOUSE_COOP_FLAGS DISCL_FOREGROUND | DISCL_NONEXCLUSIVE
#define MOUSE_CURSOR "Images/MouseCursor.png"

class DInputFramework
{
	/////////////////////////////////////////////////////////////////////////
	//	Direct Input Variables
	/////////////////////////////////////////////////////////////////////////
	IDirectInput8*				m_pInput;					//DirectInput interface
	IDirectInputDevice8*		m_pKeyboard;				//Keyboard Interface
	IDirectInputDevice8*		m_pMouse;					//Mouse Interface
	char						m_KeyboardState[256];		//Keyboard state
	char						m_PrevKeyboardState[256];	//Previous Keyboard state
	DIMOUSESTATE2				m_MouseState;				//Mouse State
	DIMOUSESTATE2				m_PrevMouseState;			//Previous Mouse State
	D3DXVECTOR2					m_MousePos;
	IDirect3DTexture9*			m_CursorTexture;

	//Private Constructor, Copy Constructor and Assignment Operator for singleton.
	DInputFramework(){}
	DInputFramework(const DInputFramework& rhs);
	DInputFramework& operator=(const DInputFramework& rhs);
public:
	~DInputFramework();

	/////////////////////////////////////////////////////////////////////////
	//	Direct Input Keyboard
	/////////////////////////////////////////////////////////////////////////
		//Parameters:
		//char key -	DIK code for the key being tested.
	
	//Returns true if key is pressed.
	bool KeyDown(char key);
	//Returns true if key is not pressed, and was pressed last update.
	bool KeyReleased(char key);
	//Returns true if key is pressed, and was pressed last update.
	bool KeyHeld(char key);
	//Returns the character for the key currently pressed.
	std::string GetText();

	/////////////////////////////////////////////////////////////////////////
	//	Direct Input Mouse
	/////////////////////////////////////////////////////////////////////////
		//Parameters:
		//int button -	Mouse button that should be tested. Supports 8 buttons.
		//				0 = Left, 1 = Right, 2 = Middle
	

	//Returns true if mouse button is pressed.
	bool MouseButtonDown(int button);
	//Returns true if mouse button is not pressed, and was pressed last update.
	bool MouseButtonReleased(int button);
	//Returns true if mouse button is pressed, and was pressed last update.
	bool MouseButtonHeld(int button);
	//Returns the mouse's position on screen.
	D3DXVECTOR2 GetMousePos(){ return m_MousePos;}
	//Returns the change in mouse position since last update
	D3DXVECTOR3 GetMouseMovement(){ return D3DXVECTOR3((float)m_MouseState.lX, (float)m_MouseState.lY, (float)m_MouseState.lZ);}

	/////////////////////////////////////////////////////////////////////////
	//	Direct Input Control
	/////////////////////////////////////////////////////////////////////////
		//Parameters:
		//hWnd - Handle to the window that owns this instance
		//hInst - Handle to the application instance.

	//Run once at startup to initialize Direct Input.
	void Init(HWND hWnd, HINSTANCE hInst);
	//Run once per update to get new input.
	void Update();
	//Run once per udpate to draw cursor and mouse position if active.
	void Render();
	//Run once at shutdown to release DirectInput.
	void Shutdown();
	//Accessor method.
	static DInputFramework* Instance();
};