#include "FMODAudio.h"

FMODAudio* FMODAudio::Instance()
{
	static FMODAudio instance;
	return &instance;
}

FMODAudio::FMODAudio()
{
	FMOD::Sound* soundIn;

	FMOD::System_Create(&m_pFMODsystem);
	m_pFMODsystem->init(100, FMOD_INIT_NORMAL, 0);

	m_pFMODsystem->createSound("Audio/Menu_Hover.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Menu_Hover.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Repair.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Repair.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Reload.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Reload.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Machinegun_Hit.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Machinegun_Hit.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Machinegun_Fire.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Machinegun_Fire.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Cannon_Hit.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Cannon_Hit.wav"] = soundIn;
	m_pFMODsystem->createSound("Audio/Cannon_Fire.wav", FMOD_DEFAULT, 0, &soundIn);
	m_SoundMap["Cannon_Fire.wav"] = soundIn;
	m_pFMODsystem->createStream("Audio/Music/Menu.mp3", FMOD_LOOP_NORMAL, 0, &soundIn);
	m_SoundMap["Menu.mp3"] = soundIn;
	m_pFMODsystem->createStream("Audio/Music/Game.mp3", FMOD_LOOP_NORMAL, 0, &soundIn);
	m_SoundMap["Game.mp3"] = soundIn;
	m_pFMODsystem->createStream("Audio/Music/Game_Lost.mp3", FMOD_LOOP_NORMAL, 0, &soundIn);
	m_SoundMap["Game_Lost.mp3"] = soundIn;

}

FMODAudio::~FMODAudio()
{
	std::map<std::string, FMOD::Sound*>::const_iterator itr;
	for(itr = m_SoundMap.begin(); itr != m_SoundMap.end(); itr++)
	{
		itr->second->release();
	}
}

void FMODAudio::Update()
{
	m_pFMODsystem->update();
}

void FMODAudio::PlaySong(std::string filename)
{
	m_pFMODsystem->playSound(FMOD_CHANNEL_FREE, m_SoundMap[filename], false, &m_pMusicChannel);
	m_pMusicChannel->setLoopCount(-1);
}

void FMODAudio::StopSong()
{
	m_pMusicChannel->stop();
}

void FMODAudio::PlaySoundFX(std::string filename)
{
	m_pFMODsystem->playSound(FMOD_CHANNEL_FREE, m_SoundMap[filename], false, &m_pFXChannel);
	m_pFXChannel->setVolume(0.5f);
}