#include "GameStateHighScore.h"
#include "DirectInput.h"
#include <fstream>
#include <string>
#include <sstream>
#include "DirectX9App.h"

GameStateHighScore::GameStateHighScore()
{
	m_ScoreData.clear();
}

GameStateHighScore::~GameStateHighScore()
{



}

GameStateHighScore* GameStateHighScore::Instance()
{
	static GameStateHighScore instance;
	return &instance;
}

void GameStateHighScore::Enter(DirectX9Game* owner)
{
	
	m_ScoreData.clear();
	//Open input file
	std::ifstream scoreFileIn(DATA_FILE);
		
	//Read in current input and save to map
	if(scoreFileIn.is_open())
	{
		std::string name;
		std::string score;
		for(int i = 0; i < 10; i++)
		{
			getline(scoreFileIn, name);
			getline(scoreFileIn, score);
			m_ScoreData[atoi(score.c_str())] = name;
		}
	}
	scoreFileIn.close();
	m_AddingScore = false;
}

void GameStateHighScore::Execute(DirectX9Game* owner, float dt)
{
	//Return to previous state on Enter
	if(gInput->KeyDown(DIK_RETURN))
	{
		owner->GetFSM()->RevertState();
	}
}


void GameStateHighScore::Exit(DirectX9Game* owner)
{
	//Write data back to file
	std::ofstream scoreFileOut("High_Score.dat", std::ios::trunc);
	std::map<int, std::string>::reverse_iterator itr;

	for(itr = m_ScoreData.rbegin(); itr != m_ScoreData.rend(); itr++)
	{
		scoreFileOut << itr->second << std::endl;
		scoreFileOut << itr->first << std::endl;
	}
	//Close data file
	scoreFileOut.close();
}


void GameStateHighScore::Render(DirectX9Game* owner)
{
	D3DXMATRIX worldMat, scaleMat, transMat;
	D3DXMatrixIdentity(&worldMat);
	//Checks if there is a new high score and adds to the list
	
	if(owner->GetScore() > m_ScoreData.begin()->first)
	{
		m_AddingScore = true;
		//Add new score
		m_ScoreData[owner->GetScore()] = m_Input.str();
		//Delete the first score (lowest)
		m_ScoreData.erase(m_ScoreData.begin());
		//Clear the input stream
		m_AddingScore = false;
		
	}
	
		
	if(!m_AddingScore)
	{


		//Draw the high score table.
		std::map<int, std::string>::reverse_iterator itr;
		std::stringstream highScoreLeft, highScoreRight;
		
		//Load the text data
		int i = 0;
		for(itr = m_ScoreData.rbegin(); itr != m_ScoreData.rend(); itr++)
		{
			if(i < 5)
				highScoreLeft << itr->second << " " << itr->first << std::endl;
			else
				highScoreRight << itr->second << " " << itr->first << std::endl;
			i++;
		}

		//Draw Left column
		D3DXMatrixTranslation(&transMat, gApplication->GetWindowSize().x / 4 + 32, gApplication->GetWindowSize().y / 4 + 32, 0);
		D3DXMatrixScaling(&scaleMat, 2.0, 2.0, 0.0);
		worldMat = scaleMat * transMat;
		gSprite->SetTransform(&worldMat);
		gApplication->GetDefaultFont()->DrawText(gSprite, highScoreLeft.str().c_str(), -1, 0, DT_TOP | DT_NOCLIP, D3DCOLOR_ARGB(255,0,0,0));

		//Draw Right column
		D3DXMatrixTranslation(&transMat, gApplication->GetWindowSize().x / 2 + 32, gApplication->GetWindowSize().y / 4 + 32, 0);
		D3DXMatrixScaling(&scaleMat, 2.0, 2.0, 0.0);
		worldMat = scaleMat * transMat;
		gSprite->SetTransform(&worldMat);
		gApplication->GetDefaultFont()->DrawText(gSprite, highScoreRight.str().c_str(), -1, 0, DT_TOP | DT_NOCLIP, D3DCOLOR_ARGB(255,0,0,0));

		//Draw Header
		D3DXMatrixTranslation(&transMat, gApplication->GetWindowSize().x / 4 - 32, gApplication->GetWindowSize().y / 8, 0);
		gSprite->SetTransform(&transMat);
		gSprite->Draw(gAssets->GetTexture("High_Score_Header.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	}
	
}


