#include "GameStateMenuGameOver.h"
#include "DirectInput.h"
#include "DirectX9Game.h"
#include "GameStatePlaying.h"
#include <sstream>
#include "FMODAudio.h"

GameStateMenuGameOver::GameStateMenuGameOver()
{



}

GameStateMenuGameOver::~GameStateMenuGameOver()
{



}

GameStateMenuGameOver* GameStateMenuGameOver::Instance()
{
	static GameStateMenuGameOver instance;
	return &instance;
}

void GameStateMenuGameOver::Enter(DirectX9Game* owner)
{
	m_Selection = GameOverNone;
	//Init the button RECTs
	m_ButtonRestart.bottom = 710;
	m_ButtonRestart.top = 640;
	m_ButtonRestart.left = 200;
	m_ButtonRestart.right = 580;

	m_ButtonQuit.bottom = 710;
	m_ButtonQuit.top = 640;
	m_ButtonQuit.left = 700;
	m_ButtonQuit.right = 1050;
	gAudio->StopSong();
	gAudio->PlaySong("Game_Lost.mp3");
}

void GameStateMenuGameOver::Execute(DirectX9Game* owner, float dt)
{
	m_SelectionLast = m_Selection;
	//Check mouse position against button rects if mouse has moved.
	if(D3DXVec3Length(&gInput->GetMouseMovement()) > 0.001f)
	{
		D3DXVECTOR2 mousePos = gInput->GetMousePos();
		if((mousePos.x > m_ButtonRestart.left) && (mousePos.x < m_ButtonRestart.right) && (mousePos.y > m_ButtonRestart.top) && (mousePos.y < m_ButtonRestart.bottom))
		{
			m_Selection = GameOverRestart;
		}

		if((mousePos.x > m_ButtonQuit.left) && (mousePos.x < m_ButtonQuit.right) && (mousePos.y > m_ButtonQuit.top) && (mousePos.y < m_ButtonQuit.bottom))
		{
			m_Selection = GameOverQuit;
		}
	}
	//Get Keyboard input to change selection
	if(gInput->KeyReleased(DIK_A) || gInput->KeyReleased(DIK_D))
	{
		if((m_Selection == GameOverNone) ||(m_Selection == GameOverQuit))
			m_Selection = GameOverRestart;
		else
			m_Selection = GameOverQuit;
	}

	if(gInput->KeyReleased(DIK_W) || gInput->KeyReleased(DIK_S))
	{
		if((m_Selection == GameOverNone) ||(m_Selection == GameOverRestart))
			m_Selection = GameOverQuit;
		else
			m_Selection = GameOverRestart;
	}

	if(m_Selection != m_SelectionLast)
		gAudio->PlaySoundFX("Menu_Hover.wav");

	//If mouse clicked or enter pressed and button selected, process that selection
	if(gInput->KeyReleased(DIK_RETURN)|| gInput->MouseButtonDown(0))
	{
		gAudio->PlaySoundFX("Cannon_Hit.wav");
		if(m_Selection == GameOverQuit)
			PostQuitMessage(0);
		else if(m_Selection == GameOverRestart)
			owner->GetFSM()->ChangeState(GameStateMenuMain::Instance());
	}

}


void GameStateMenuGameOver::Exit(DirectX9Game* owner)
{
	

}


void GameStateMenuGameOver::Render(DirectX9Game* owner)
{
	//Draw button textures based on selection.
	gInput->Render();
	D3DXMATRIX worldMat, scaleMat;

	//Draw Background
	D3DXMatrixScaling(&scaleMat, 0.7f, 1.0f, 1.0f);
	D3DXMatrixTranslation(&worldMat, -40.0f, 0.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	gSprite->Draw(gAssets->GetTexture("Tank_Background.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();
	D3DXMatrixScaling(&scaleMat, 2.0f, 2.0f, 2.0f);

	//Credits Button
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x  / 8.0f, 2.5 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == GameOverRestart)
		gSprite->Draw(gAssets->GetTexture("Main_Menu_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("Main_Menu.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//High Score Button
	D3DXMatrixTranslation(&worldMat, 4 * gApplication->GetWindowSize().x  / 8.0f, 2.5 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == GameOverQuit)
		gSprite->Draw(gAssets->GetTexture("Exit_Game_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("Exit_Game.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//Text
	D3DXMatrixScaling(&scaleMat, 1.0f, 1.0f, 1.0f);
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x  / 8.0f, gApplication->GetWindowSize().y / 6.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	gSprite->Draw(gAssets->GetTexture("Game_Over_Text.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//Render Score
	D3DXMatrixScaling(&scaleMat, 3.0f, 3.0f, 3.0f);
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x  / 2.0f, gApplication->GetWindowSize().y / 2.5f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	std::stringstream score;
	score << DirectX9Game::Instance()->GetScore();
	gApplication->GetDefaultFont()->DrawText(gSprite, score.str().c_str(), -1, 0, DT_TOP | DT_NOCLIP, D3DCOLOR_ARGB(255,0,0,0));
}