#include "GameStateMenuMain.h"
#include "DirectInput.h"
#include "DirectX9Game.h"
#include "GameStatePlayerGuide.h"
#include "GameStateMenuCredits.h"
#include "GameStateHighScore.h"
#include "AssetManager.h"
#include "FMODAudio.h"

GameStateMenuMain::GameStateMenuMain()
{



}

GameStateMenuMain::~GameStateMenuMain()
{



}

GameStateMenuMain* GameStateMenuMain::Instance()
{
	static GameStateMenuMain instance;
	return &instance;
}

void GameStateMenuMain::Enter(DirectX9Game* owner)
{
	gAssets->LoadMenuAssets();
	owner->ResetGame();
	m_Selection = MainNone;
	//Init the button RECTs
	m_ButtonCredits.bottom = 730;
	m_ButtonCredits.top = 630;
	m_ButtonCredits.left = 340;
	m_ButtonCredits.right = 600;

	/*
	m_ButtonHighScore.bottom = 760;
	m_ButtonHighScore.top = 670;
	m_ButtonHighScore.left = 700;
	m_ButtonHighScore.right = 1050;
	*/

	m_ButtonStart.bottom = 600;
	m_ButtonStart.top = 510;
	m_ButtonStart.left = 260;
	m_ButtonStart.right = 690;

	m_ButtonQuit.bottom = 630;
	m_ButtonQuit.top = 510;
	m_ButtonQuit.left = 700;
	m_ButtonQuit.right = 1050;

	gAudio->StopSong();
	gAudio->PlaySong("Menu.mp3");
}

void GameStateMenuMain::Execute(DirectX9Game* owner, float dt)
{
	m_SelectionLast = m_Selection;
	//Check mouse position against button rects if mouse has moved.
	if(D3DXVec3Length(&gInput->GetMouseMovement()) > 0.001f)
	{
		D3DXVECTOR2 mousePos = gInput->GetMousePos();

		if((mousePos.x > m_ButtonStart.left) && (mousePos.x < m_ButtonStart.right) && (mousePos.y > m_ButtonStart.top) && (mousePos.y < m_ButtonStart.bottom))
		{
			m_Selection = MainMenuSelection::MainStart;
		}

		if((mousePos.x > m_ButtonCredits.left) && (mousePos.x < m_ButtonCredits.right) && (mousePos.y > m_ButtonCredits.top) && (mousePos.y < m_ButtonCredits.bottom))
		{
			m_Selection = MainMenuSelection::MainCredits;
		}

		if((mousePos.x > m_ButtonQuit.left) && (mousePos.x < m_ButtonQuit.right) && (mousePos.y > m_ButtonQuit.top) && (mousePos.y < m_ButtonQuit.bottom))
		{
			m_Selection = MainMenuSelection::MainQuit;
		}

	/*	if((mousePos.x > m_ButtonHighScore.left) && (mousePos.x < m_ButtonHighScore.right) && (mousePos.y > m_ButtonHighScore.top) && (mousePos.y < m_ButtonHighScore.bottom))
			m_Selection = MainMenuSelection::MainHighScore; */
	}
	//Get Keyboard input to change selection
	
	if(gInput->KeyReleased(DIK_A))
	{
		switch(m_Selection)
		{
		case MainMenuSelection::MainNone:
			m_Selection = MainMenuSelection::MainQuit;
			break;
		case MainMenuSelection::MainStart:
			m_Selection = MainMenuSelection::MainQuit;
			break;
		case MainMenuSelection::MainCredits:
			m_Selection = MainMenuSelection::MainHighScore;
			break;
		case MainMenuSelection::MainQuit:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainHighScore:
			m_Selection = MainMenuSelection::MainCredits;
			break;
		}
	}

	if(gInput->KeyReleased(DIK_D))
	{
		switch(m_Selection)
		{
		case MainMenuSelection::MainNone:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainStart:
			m_Selection = MainMenuSelection::MainQuit;
			break;
		case MainMenuSelection::MainCredits:
			m_Selection = MainMenuSelection::MainHighScore;
			break;
		case MainMenuSelection::MainQuit:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainHighScore:
			m_Selection = MainMenuSelection::MainCredits;
			break;
		}
	}

	
	if(gInput->KeyReleased(DIK_W))
	{
		switch(m_Selection)
		{
		case MainMenuSelection::MainNone:
			m_Selection = MainMenuSelection::MainCredits;
			break;
		case MainMenuSelection::MainStart:
			m_Selection = MainMenuSelection::MainCredits;
			break;
		case MainMenuSelection::MainCredits:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainQuit:
			m_Selection = MainMenuSelection::MainHighScore;
			break;
		case MainMenuSelection::MainHighScore:
			m_Selection = MainMenuSelection::MainQuit;
			break;
		}
	}

	if(gInput->KeyReleased(DIK_S))
	{
		switch(m_Selection)
		{
		case MainMenuSelection::MainNone:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainStart:
			m_Selection = MainMenuSelection::MainCredits;
			break;
		case MainMenuSelection::MainCredits:
			m_Selection = MainMenuSelection::MainStart;
			break;
		case MainMenuSelection::MainQuit:
			m_Selection = MainMenuSelection::MainHighScore;
			break;
		case MainMenuSelection::MainHighScore:
			m_Selection = MainMenuSelection::MainQuit;
			break;
		}
	}

	if(m_Selection != m_SelectionLast)
		gAudio->PlaySoundFX("Menu_Hover.wav");


	//If left mouse clicked or enter pressed and button selected, process that selection
	if(gInput->KeyDown(DIK_RETURN) || gInput->MouseButtonDown(0))
	{
		gAudio->PlaySoundFX("Cannon_Hit.wav");
		switch(m_Selection)
		{
		case MainMenuSelection::MainQuit:
			PostQuitMessage(0);
			break;
		case MainMenuSelection::MainStart:
			owner->GetFSM()->ChangeState(GameStatePlayerGuide::Instance());
			break;
		case MainMenuSelection::MainCredits:
			owner->GetFSM()->ChangeState(GameStateMenuCredits::Instance());
			break;
		case MainMenuSelection::MainHighScore:
			owner->GetFSM()->ChangeState(GameStateHighScore::Instance());
			break;
		}
	}

}


void GameStateMenuMain::Exit(DirectX9Game* owner)
{

}


void GameStateMenuMain::Render(DirectX9Game* owner)
{

	D3DXMATRIX worldMat, scaleMat;

	//Draw Background
	D3DXMatrixScaling(&scaleMat, 0.7f, 1.0f, 1.0f);
	D3DXMatrixTranslation(&worldMat, -40.0f, 0.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	gSprite->Draw(gAssets->GetTexture("Tank_Background.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//Draw Title
	D3DXMatrixScaling(&scaleMat, 1.0f, 1.0f, 1.0f);
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x / 8.0f, gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	gSprite->Draw(gAssets->GetTexture("Main_Menu_Title.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	
	gSprite->Flush();

	//Draw button textures based on selection.

	//Start Button

	D3DXMatrixScaling(&scaleMat, 2.0f, 2.0f, 2.0f);
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x / 8.0f, 2 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == MainStart)
		gSprite->Draw(gAssets->GetTexture("Start_Game_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("Start_Game.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//Quit Button
	D3DXMatrixTranslation(&worldMat, 4 * gApplication->GetWindowSize().x  / 8.0f, 2 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == MainQuit)
		gSprite->Draw(gAssets->GetTexture("Exit_Game_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("Exit_Game.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	//Credits Button
	D3DXMatrixTranslation(&worldMat, gApplication->GetWindowSize().x  / 8.0f, 2.5 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == MainCredits)
		gSprite->Draw(gAssets->GetTexture("Credits_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("Credits.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();

	/*
	//High Score Button
	D3DXMatrixTranslation(&worldMat, 4 * gApplication->GetWindowSize().x  / 8.0f, 2.5 * gApplication->GetWindowSize().y / 4.0f, 0.0f);
	worldMat = scaleMat * worldMat;
	gSprite->SetTransform(&worldMat);
	if(m_Selection == MainHighScore)
		gSprite->Draw(gAssets->GetTexture("High_Score_Selected.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	else
		gSprite->Draw(gAssets->GetTexture("High_Score.png"), 0, 0, 0, D3DCOLOR_ARGB(255,255,255,255));
	gSprite->Flush();
	*/

	gInput->Render();

}