#include "Messenger.h"
#include "ActorManager.h"
#include "Defines.h"
#include "Messages.h"
#include "DirectX9App.h"

Messenger* Messenger::Instance()
{
	static Messenger instance;
	return &instance;
}

void Messenger::Update(float dt)
{
	//Local vector that stores the index of messages being sent, so they can be cleaned up.
	std::vector<Telegram> validMessages;

	//Update message timers
	for(unsigned i = 0; i < m_PendingMessages.size(); ++i)
	{
		m_PendingMessages[i].delay -= dt;

		//Send message if time is up
		if(m_PendingMessages[i].delay <= 0.0f)
		{
			Actor* receiver = gActors->GetActor(m_PendingMessages[i].receiver);
			if(receiver)
				receiver->ReceiveMessage(m_PendingMessages[i]);
			//Add the index to the list of messages to kill at the end of this tick.
		}
		else
		{
			//Add any messages still waiting to the list to keep
			validMessages.push_back(m_PendingMessages[i]);
		}
	}

	//Clear message list and assign the valid messages to it.
	m_PendingMessages.clear();
	m_PendingMessages = validMessages;

}

void Messenger::AddMessage(int sender, int receiver, int msg, float delay)
{
	//Create the new message
	Telegram newMessage;
	newMessage.receiver = receiver;
	newMessage.sender = sender;
	newMessage.msg = msg;
	newMessage.delay = delay;
	//Add the message to the list
	m_PendingMessages.push_back(newMessage);
}

void Messenger::Reset()
{
	m_PendingMessages.clear();
}

