#include "Physics.h"
#include "D3DXMathFunctions.h"
#include "DirectX9App.h"

void Physics::Init(D3DXVECTOR3 position, D3DXVECTOR3 heading, float contactRadius, float maxAccel, float maxSpeed, float maxRotationSpeed, D3DXVECTOR3 gravity)
{
	m_Position = position;
	m_Velocity = D3DXZEROVEC3;
	m_Acceleration = D3DXZEROVEC3;
	m_Gravity = gravity;
	m_ContactRadius = contactRadius;
	m_MaxAccel = maxAccel;
	m_MaxSpeed = maxSpeed;
	m_FrictionForce = 120.0f;
	m_MaxRotationSpeed = maxRotationSpeed;
}


void Physics::Update(float dt)
{
	//Add Gravity
	Accelerate(m_Gravity);

	//Update position
	m_Position += m_Velocity * dt;
	
	//Update Acceleration
	if(D3DXVec3Length(&m_Acceleration) > m_MaxAccel)
	{
		D3DXVec3Normalize(&m_Acceleration, &m_Acceleration);
		m_Acceleration *= m_MaxAccel;
	}

	//Apply friction force if there is a velocity
	if(D3DXVec3Length(&m_Velocity) > 0.001)
	{
		D3DXVECTOR3 frictionDirection;
		D3DXVec3Normalize(&frictionDirection, &m_Velocity);
		frictionDirection *= -m_FrictionForce;
		Accelerate(frictionDirection);
	}

	//Update velocity
	m_Velocity += m_Acceleration * dt;
	if(D3DXVec3Length(&m_Velocity) > m_MaxSpeed)
	{
		D3DXVec3Normalize(&m_Velocity, &m_Velocity);
		m_Velocity *= m_MaxSpeed;
	}

	//Reset Acceleration
	m_Acceleration = D3DXZEROVEC3;

}

void Physics::Stop()
{
	m_Acceleration = D3DXZEROVEC3;
	m_Velocity = D3DXZEROVEC3;
}
