#include "Pickups.h"
#include "Messages.h"
#include "Defines.h"
#include "DirectX9App.h"
#include "D3DXMathFunctions.h"
#include "Messenger.h"
#include "AssetManager.h"
#include "ActorManager.h"
#include "Camera.h"
#include <ctime>
#include "PlayerTank.h"
#include "FMODAudio.h"


AmmoPickup::AmmoPickup(int ID):Actor(ID)
{
	ZeroMemory(&m_Material, sizeof(m_Material));

	m_Material.Ambient = D3DXCOLOR(0.2f, 0.2f, 0.2f, 1.0f);		// Ambient color reflected
	m_Material.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);		// Diffuse color reflected
	m_Material.Emissive = D3DXCOLOR(0.0f, 0.0f, 0.0f, 1.0f);		// Emissive color reflected
	m_Material.Specular = D3DXCOLOR(0.8f, 0.8f, 0.8f, 0.8f);		// Specular
	m_Material.Power = 50000.0f;									// Specular highlight intensity
	
	m_hTech = gAssets->GetShader("ProjectB.fx")->GetTechniqueByName("tech0");
	
	m_AmountReloaded= 25;
	m_Timeout = 15.0f;
	m_CooldownTime = 0.0f;
	m_IsActive = true;
	m_ContactRadius = 32.0f;
	m_Rotation = 0.0f;
}

void AmmoPickup::Update(float dt)
{
	if(m_CooldownTime > 0.0)
	{
		m_CooldownTime -= dt;
		m_IsActive = false;
		m_Position.y = 0.0f;
		m_Position.x = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
		m_Position.z = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
	}
	else
		m_IsActive = true;
	m_Rotation += dt;

	PlayerTank* player = (PlayerTank*)gActors->GetPlayer();
	float distanceToPlayer = D3DXVec3Length(&(player->GetTank()->GetPhysics()->GetPosition() - m_Position));
	if((distanceToPlayer <= (player->GetTank()->GetPhysics()->GetContactRadius() + m_ContactRadius)) && (m_IsActive))
	{
		player->GetTank()->ReloadMachineGuns(25);
		player->GetTank()->ReloadMainGun(5);
		m_CooldownTime = m_Timeout;
		gAudio->PlaySoundFX("Reload.wav");
	}
}

void AmmoPickup::Render()
{
	if(m_IsActive)
	{
		D3DXMATRIX transMat, rotMat, scaleMat, worldMat, WIT;
		D3DXMatrixIdentity(&transMat);
		D3DXMatrixIdentity(&scaleMat);
		D3DXMatrixIdentity(&rotMat);
		D3DXMatrixIdentity(&worldMat);
	
		gAssets->GetShader("ProjectB.fx")->SetTechnique(m_hTech);

		//Get light and camera data
		D3DLIGHT9 mainLight = gActors->GetMainLight();
		D3DXMATRIX viewMat = ((Camera*)gActors->GetActiveCamera())->GetView();
		D3DXMATRIX projMat = ((Camera*)gActors->GetActiveCamera())->GetProjection();
		D3DXVECTOR3 camPosition = ((Camera*)gActors->GetActiveCamera())->GetPosition();

		gAssets->GetShader("ProjectB.fx")->SetFloatArray("eyePos", (float*)camPosition, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightPos", (float*)&mainLight.Position, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("ambientLight", (float*)&mainLight.Ambient, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("specularLight", (float*)&mainLight.Specular, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("diffuseLight", (float*)&mainLight.Diffuse, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightAttenuation", (float*)&D3DXVECTOR3(mainLight.Attenuation0, mainLight.Attenuation1, mainLight.Attenuation2), 3);

	
		// Begin Effect
		UINT numPasses = 0;
		gAssets->GetShader("ProjectB.fx")->Begin(&numPasses, 0);

		for(UINT i = 0; i < numPasses; ++i)
		{
			gAssets->GetShader("ProjectB.fx")->BeginPass(i);
	
	
			// Calculate Matrix Transform
			D3DXMatrixScaling(&scaleMat, 3.5f, 3.5f, 3.5f);			// Scaling
			D3DXMatrixRotationYawPitchRoll(&rotMat, m_Rotation, 0.0f, 0.0f); // Rotation on Yaw, Pitch, and Roll
			D3DXMatrixTranslation(&transMat, m_Position.x, m_Position.y, m_Position.z);		// Translation
			D3DXMatrixMultiply(&scaleMat, &scaleMat, &rotMat);		// Multiply scale and rotation, store in scale
			D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);	// Multiply scale and translation, store in world
		
			D3DXMatrixInverse(&WIT, 0, &worldMat);
			D3DXMatrixTranspose(&WIT, &WIT);
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);
			gAssets->GetShader("ProjectB.fx")->SetValue("ambientMaterial", &m_Material.Ambient, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetValue("diffuseMaterial", &m_Material.Diffuse, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetValue("specularMaterial", &m_Material.Specular, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetFloat("specularPower", m_Material.Power);
			gAssets->GetShader("ProjectB.fx")->SetTexture("tex", gAssets->GetTexture("TankRound.png"));

			gAssets->GetShader("ProjectB.fx")->CommitChanges();
		
			gAssets->GetMesh("Ammo.x")->DrawSubset(0);
					
			gAssets->GetShader("ProjectB.fx")->EndPass();
		}

	gAssets->GetShader("ProjectB.fx")->End();
	
	}
}

void AmmoPickup::ReceiveMessage(Telegram incoming)
{
	if(incoming.msg == Message::msg_Reset)
	{
		m_CooldownTime = m_Timeout;
	}
}

RepairPickup::RepairPickup(int ID):Actor(ID)
{
	ZeroMemory(&m_Material, sizeof(m_Material));

	m_Material.Ambient = D3DXCOLOR(0.2f, 0.2f, 0.2f, 1.0f);		// Ambient color reflected
	m_Material.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);		// Diffuse color reflected
	m_Material.Emissive = D3DXCOLOR(0.0f, 0.0f, 0.0f, 1.0f);		// Emissive color reflected
	m_Material.Specular = D3DXCOLOR(0.8f, 0.8f, 0.8f, 0.8f);		// Specular
	m_Material.Power = 50000.0f;									// Specular highlight intensity
	
	m_hTech = gAssets->GetShader("ProjectB.fx")->GetTechniqueByName("tech0");

	m_AmountHealed = 25;
	m_Timeout = 15.0f;
	m_CooldownTime = 0.0f;
	m_IsActive = true;
	m_ContactRadius = 32.0f;
	m_Rotation = 0.0f;
}

void RepairPickup::Update(float dt)
{
	if(m_CooldownTime > 0.0)
	{
		m_CooldownTime -= dt;
		m_IsActive = false;
		m_Position.y = 0.0f;
		m_Position.x = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
		m_Position.z = rand()%2 > 0 ? (float)(rand()%150) : (float)(rand()%150) * -1.0f;
	}
	else
		m_IsActive = true;
	m_Rotation += dt;

	PlayerTank* player = (PlayerTank*)gActors->GetPlayer();
	float distanceToPlayer = D3DXVec3Length(&(player->GetTank()->GetPhysics()->GetPosition() - m_Position));
	if((distanceToPlayer <= (player->GetTank()->GetPhysics()->GetContactRadius() + m_ContactRadius)) && (m_IsActive))
	{
		player->AdjustHealth(20);
		m_CooldownTime = m_Timeout;
		gAudio->PlaySoundFX("Repair.wav");
	}

}

void RepairPickup::Render()
{
	if(m_IsActive)
	{
		D3DXMATRIX transMat, rotMat, scaleMat, worldMat, WIT;
		D3DXMatrixIdentity(&transMat);
		D3DXMatrixIdentity(&scaleMat);
		D3DXMatrixIdentity(&rotMat);
		D3DXMatrixIdentity(&worldMat);
	
		gAssets->GetShader("ProjectB.fx")->SetTechnique(m_hTech);

		//Get light and camera data
		D3DLIGHT9 mainLight = gActors->GetMainLight();
		D3DXMATRIX viewMat = ((Camera*)gActors->GetActiveCamera())->GetView();
		D3DXMATRIX projMat = ((Camera*)gActors->GetActiveCamera())->GetProjection();
		D3DXVECTOR3 camPosition = ((Camera*)gActors->GetActiveCamera())->GetPosition();

		gAssets->GetShader("ProjectB.fx")->SetFloatArray("eyePos", (float*)camPosition, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightPos", (float*)&mainLight.Position, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("ambientLight", (float*)&mainLight.Ambient, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("specularLight", (float*)&mainLight.Specular, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("diffuseLight", (float*)&mainLight.Diffuse, 3);
		gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightAttenuation", (float*)&D3DXVECTOR3(mainLight.Attenuation0, mainLight.Attenuation1, mainLight.Attenuation2), 3);

	
		// Begin Effect
		UINT numPasses = 0;
		gAssets->GetShader("ProjectB.fx")->Begin(&numPasses, 0);

		for(UINT i = 0; i < numPasses; ++i)
		{
			gAssets->GetShader("ProjectB.fx")->BeginPass(i);
	
	
			// Calculate Matrix Transform
			D3DXMatrixScaling(&scaleMat, 1.7f, 1.7f, 1.7f);			// Scaling
			D3DXMatrixRotationYawPitchRoll(&rotMat, m_Rotation, 0.0f, 0.0f); // Rotation on Yaw, Pitch, and Roll
			D3DXMatrixTranslation(&transMat, m_Position.x, m_Position.y, m_Position.z);		// Translation
			D3DXMatrixMultiply(&scaleMat, &scaleMat, &rotMat);		// Multiply scale and rotation, store in scale
			D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);	// Multiply scale and translation, store in world
		
			D3DXMatrixInverse(&WIT, 0, &worldMat);
			D3DXMatrixTranspose(&WIT, &WIT);
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
			gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);
			gAssets->GetShader("ProjectB.fx")->SetValue("ambientMaterial", &m_Material.Ambient, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetValue("diffuseMaterial", &m_Material.Diffuse, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetValue("specularMaterial", &m_Material.Specular, sizeof(D3DXCOLOR));
			gAssets->GetShader("ProjectB.fx")->SetFloat("specularPower", m_Material.Power);
			gAssets->GetShader("ProjectB.fx")->SetTexture("tex", gAssets->GetTexture("WrenchMetal.png"));

			gAssets->GetShader("ProjectB.fx")->CommitChanges();
		
			gAssets->GetMesh("Wrench.x")->DrawSubset(0);
					
			gAssets->GetShader("ProjectB.fx")->EndPass();
		}

	gAssets->GetShader("ProjectB.fx")->End();
	
	}
}

void RepairPickup::ReceiveMessage(Telegram incoming)
{
	if(incoming.msg == Message::msg_Reset)
	{
		m_CooldownTime = m_Timeout;
	}
}