#include "PlayerTank.h"
#include "Defines.h"
#include "DirectInput.h"
#include "D3DXMathFunctions.h"
#include "ActorManager.h"
#include "Camera.h"
#include "DirectX9App.h"
#include "AITank.h"
#include "ActorFactory.h"
#include "DirectX9Game.h"
#include "GameStateMenuGameOver.h"

PlayerTank::PlayerTank(int ID):Actor(ID),m_Tank(ID),m_Health(100)
{

}


void PlayerTank::Update(float dt)
{
	
	if(m_Health > m_MaxHealth)
	{
		m_Health = m_MaxHealth;
	}
	else if (m_Health < 0)
	{
		DirectX9Game::Instance()->GetFSM()->ChangeState(GameStateMenuGameOver::Instance());
	}
	//Process Input
		
	//See if mouse has moved
	if(D3DXVec3Length(&gInput->GetMouseMovement()) != 0.0f)
	{
		//Rotate turret on mouse X
		((Camera*)gActors->GetActiveCamera())->Yaw(gInput->GetMouseMovement().x * dt * m_Tank.GetPhysics()->GetMaxRotationSpeed() * 0.5f);
		m_Tank.RotateTurret(gInput->GetMouseMovement().x * dt * m_Tank.GetPhysics()->GetMaxRotationSpeed() * 0.5f);
	}

	//Rotate Base on A/D
	if(gInput->KeyDown(DIK_D))
	{
		m_Tank.RotateBase(dt * m_Tank.GetPhysics()->GetMaxRotationSpeed());
	}

	if(gInput->KeyDown(DIK_A))
	{
		m_Tank.RotateBase(-dt * m_Tank.GetPhysics()->GetMaxRotationSpeed());
	}
	//Accelerate Base on W/S
	if(gInput->KeyDown(DIK_W))
	{
		m_Tank.GetPhysics()->Accelerate(m_Tank.GetBaseDirection() * -m_Tank.GetPhysics()->GetMaxAccel());
	}
	if(gInput->KeyDown(DIK_S))
	{
		m_Tank.GetPhysics()->Accelerate(m_Tank.GetBaseDirection() * m_Tank.GetPhysics()->GetMaxAccel());
	}

	//Fire Control
	if(gInput->KeyDown(DIK_SPACE))
	{
		m_Tank.FireMainGun();
	}
	if((gInput->KeyDown(DIK_LCONTROL) || (gInput->KeyDown(DIK_RCONTROL))))
	{
		m_Tank.FireMachineGuns();
	}

	//Update physics
	m_Tank.Update(dt);

	//Contact with enemies
	AITank* enemy1 = (AITank*)gActors->GetActiveAITank();
	if(enemy1 != nullptr)
	{
		D3DXVECTOR3 direction1 = m_Tank.GetPhysics()->GetPosition() - enemy1->GetTank()->GetPhysics()->GetPosition();
		float distance1 = D3DXVec3Length(&direction1);
		float contact1 = m_Tank.GetPhysics()->GetContactRadius() + enemy1->GetTank()->GetPhysics()->GetContactRadius();
		if(distance1 <= contact1)
		{
			D3DXVECTOR3 moveDirection, newPosition;
			D3DXVec3Normalize(&moveDirection, &direction1);
			newPosition = (m_Tank.GetPhysics()->GetPosition()) + (moveDirection * (contact1 - distance1) * 1.2f);
			newPosition.y = 0;
			m_Tank.GetPhysics()->SetPosition(newPosition);
			m_Tank.GetPhysics()->Stop();
		}
	}
	//Set Cannon
	

	//Update camera position based on tank position
	D3DXVECTOR3 camPosition;
	camPosition = m_Tank.GetPhysics()->GetPosition() + D3DXVECTOR3(0, 3, 0) + (((Camera*)gActors->GetActiveCamera())->GetZAxis() * -12.0f);
	((Camera*)gActors->GetActiveCamera())->SetPosition(camPosition);

	
}


void PlayerTank::Render()
{
	m_Tank.Render();
}

void PlayerTank::ReceiveMessage(Telegram incoming)
{
	switch(incoming.msg)
	{
	case msg_Destroy:
		if(incoming.sender = m_ID)
			gActors->RemoveActor(m_ID);
	}
}

const int PlayerTank::m_MaxHealth = 100;