#include "SkyBox.h"
#include "ActorManager.h"
#include "Camera.h"
#include "AssetManager.h"

SkyBox::SkyBox(int ID):Actor(ID)
{
	m_hTech = gAssets->GetShader("ProjectB.fx")->GetTechniqueByName("tech0");

	ZeroMemory(&m_Material, sizeof(m_Material));

	m_Material.Ambient = D3DXCOLOR(0.2f, 0.2f, 0.2f, 1.0f);		// Ambient color reflected
	m_Material.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);		// Diffuse color reflected
	m_Material.Emissive = D3DXCOLOR(0.0f, 0.0f, 0.0f, 1.0f);		// Emissive color reflected
	m_Material.Specular = D3DXCOLOR(0.8f, 0.8f, 0.8f, 0.8f);		// Specular
	m_Material.Power = 50000.0f;

	//////////////////////////////////////////////////////////////////////////
	// Vertex Declaration
	//////////////////////////////////////////////////////////////////////////
	// Vertex declaration
	D3DVERTEXELEMENT9 decl[] =
	{
		{0, 0, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0},
		{0, 12, D3DDECLTYPE_FLOAT3, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_NORMAL, 0},
		{0, 24, D3DDECLTYPE_FLOAT2, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0},
		D3DDECL_END()
	};
	// Create vertex declaration
	gD3DDevice->CreateVertexDeclaration(decl, &m_D3DVertexDecl);

	//////////////////////////////////////////////////////////////////////////
	// Load world cube data (verts and indices) into respective arrays
	//////////////////////////////////////////////////////////////////////////
	// Load vertex info

	// Front
	m_CubeVerts[0].position = D3DXVECTOR3(-1.0f, -1.0f, -1.0f);
	m_CubeVerts[1].position = D3DXVECTOR3(-1.0f, 1.0f, -1.0f);
	m_CubeVerts[2].position = D3DXVECTOR3(1.0f, 1.0f, -1.0f);
	m_CubeVerts[3].position = D3DXVECTOR3(1.0f, -1.0f, -1.0f);
	D3DXVec3Normalize(&m_CubeVerts[0].normal, &D3DXVECTOR3(0.0f, 0.0f, 1.0f));
	D3DXVec3Normalize(&m_CubeVerts[1].normal, &D3DXVECTOR3(0.0f, 0.0f, 1.0f));
	D3DXVec3Normalize(&m_CubeVerts[2].normal, &D3DXVECTOR3(0.0f, 0.0f, 1.0f));
	D3DXVec3Normalize(&m_CubeVerts[3].normal, &D3DXVECTOR3(0.0f, 0.0f, 1.0f));
	m_CubeVerts[0].uv = D3DXVECTOR2(0.5f, 0.5f);
	m_CubeVerts[1].uv = D3DXVECTOR2(0.5f, 0.0f);
	m_CubeVerts[2].uv = D3DXVECTOR2(0.75f, 0.0f);
	m_CubeVerts[3].uv = D3DXVECTOR2(0.75f, 0.5f);

	// Back
	m_CubeVerts[4].position = D3DXVECTOR3(-1.0f, -1.0f, 1.0f);
	m_CubeVerts[5].position = D3DXVECTOR3(1.0f, -1.0f, 1.0f);
	m_CubeVerts[6].position = D3DXVECTOR3(1.0f, 1.0f, 1.0f);
	m_CubeVerts[7].position = D3DXVECTOR3(-1.0f, 1.0f, 1.0f);
	D3DXVec3Normalize(&m_CubeVerts[4].normal, &D3DXVECTOR3(0.0f, 0.0f, -1.0f));
	D3DXVec3Normalize(&m_CubeVerts[5].normal, &D3DXVECTOR3(0.0f, 0.0f, -1.0f));
	D3DXVec3Normalize(&m_CubeVerts[6].normal, &D3DXVECTOR3(0.0f, 0.0f, -1.0f));
	D3DXVec3Normalize(&m_CubeVerts[7].normal, &D3DXVECTOR3(0.0f, 0.0f, -1.0f));
	m_CubeVerts[4].uv = D3DXVECTOR2(0.75f, 1.0f);
	m_CubeVerts[5].uv = D3DXVECTOR2(0.5f, 1.0f);
	m_CubeVerts[6].uv = D3DXVECTOR2(0.5f, 0.5f);
	m_CubeVerts[7].uv = D3DXVECTOR2(0.75f, 0.5f);

	// Top
	m_CubeVerts[8].position = D3DXVECTOR3(-1.0f, 1.0f, -1.0f);
	m_CubeVerts[9].position = D3DXVECTOR3(-1.0f, 1.0f, 1.0f);
	m_CubeVerts[10].position = D3DXVECTOR3(1.0f, 1.0f, 1.0f);
	m_CubeVerts[11].position = D3DXVECTOR3(1.0f, 1.0f, -1.0f);
	D3DXVec3Normalize(&m_CubeVerts[8].normal, &D3DXVECTOR3(0.0f, -1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[9].normal, &D3DXVECTOR3(0.0f, -1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[10].normal, &D3DXVECTOR3(0.0f, -1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[11].normal, &D3DXVECTOR3(0.0f, -1.0f, 0.0f));
	m_CubeVerts[8].uv = D3DXVECTOR2(0.0f, 0.5f);
	m_CubeVerts[9].uv = D3DXVECTOR2(0.0f, 0.0f);
	m_CubeVerts[10].uv = D3DXVECTOR2(0.25f, 0.0f);
	m_CubeVerts[11].uv = D3DXVECTOR2(0.25f, 0.5f);

	// Bottom
	m_CubeVerts[12].position = D3DXVECTOR3(-1.0f, -1.0f, -1.0f);
	m_CubeVerts[13].position = D3DXVECTOR3(1.0f, -1.0f, -1.0f);
	m_CubeVerts[14].position = D3DXVECTOR3(1.0f, -1.0f, 1.0f);
	m_CubeVerts[15].position = D3DXVECTOR3(-1.0f, -1.0f, 1.0f);
	D3DXVec3Normalize(&m_CubeVerts[12].normal, &D3DXVECTOR3(0.0f, 1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[13].normal, &D3DXVECTOR3(0.0f, 1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[14].normal, &D3DXVECTOR3(0.0f, 1.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[15].normal, &D3DXVECTOR3(0.0f, 1.0f, 0.0f));
	m_CubeVerts[12].uv = D3DXVECTOR2(0.0f, 1.0f);
	m_CubeVerts[13].uv = D3DXVECTOR2(0.0f, 0.5f);
	m_CubeVerts[14].uv = D3DXVECTOR2(0.25f, 0.5f);
	m_CubeVerts[15].uv = D3DXVECTOR2(0.25f, 1.0f);
	
	// Left
	m_CubeVerts[16].position = D3DXVECTOR3(-1.0f, -1.0f, 1.0f);
	m_CubeVerts[17].position = D3DXVECTOR3(-1.0f, 1.0f, 1.0f);
	m_CubeVerts[18].position = D3DXVECTOR3(-1.0f, 1.0f, -1.0f);
	m_CubeVerts[19].position = D3DXVECTOR3(-1.0f, -1.0f, -1.0f);
	D3DXVec3Normalize(&m_CubeVerts[16].normal, &D3DXVECTOR3(1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[17].normal, &D3DXVECTOR3(1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[18].normal, &D3DXVECTOR3(1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[19].normal, &D3DXVECTOR3(1.0f, 0.0f, 0.0f));
	m_CubeVerts[16].uv = D3DXVECTOR2(0.25f, 0.5f);
	m_CubeVerts[17].uv = D3DXVECTOR2(0.25f, 0.0f);
	m_CubeVerts[18].uv = D3DXVECTOR2(0.5f, 0.0f);
	m_CubeVerts[19].uv = D3DXVECTOR2(0.5f, 0.5f);

	// Right
	m_CubeVerts[20].position = D3DXVECTOR3(1.0f, -1.0f, -1.0f);
	m_CubeVerts[21].position = D3DXVECTOR3(1.0f, 1.0f, -1.0f);
	m_CubeVerts[22].position = D3DXVECTOR3(1.0f, 1.0f, 1.0f);
	m_CubeVerts[23].position = D3DXVECTOR3(1.0f, -1.0f, 1.0f);
	D3DXVec3Normalize(&m_CubeVerts[20].normal, &D3DXVECTOR3(-1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[21].normal, &D3DXVECTOR3(-1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[22].normal, &D3DXVECTOR3(-1.0f, 0.0f, 0.0f));
	D3DXVec3Normalize(&m_CubeVerts[23].normal, &D3DXVECTOR3(-1.0f, 0.0f, 0.0f));
	m_CubeVerts[20].uv = D3DXVECTOR2(0.25f, 1.0f);
	m_CubeVerts[21].uv = D3DXVECTOR2(0.25f, 0.5f);
	m_CubeVerts[22].uv = D3DXVECTOR2(0.5f, 0.5f);
	m_CubeVerts[23].uv = D3DXVECTOR2(0.5f, 1.0f);

	// Load index info, refers into index into verts array to compose triangles
	// Note: A clockwise winding order of verts will show the front face.

	// Front
	m_CubeIndices[0] = 2;  m_CubeIndices[1] = 1;  m_CubeIndices[2] = 0;		// Triangle 0
	m_CubeIndices[3] = 3;  m_CubeIndices[4] = 2;  m_CubeIndices[5] = 0;		// Triangle 1
	// Back
	m_CubeIndices[6] = 6;  m_CubeIndices[7] = 5;  m_CubeIndices[8] = 4;		// Triangle 2
	m_CubeIndices[9] = 7;  m_CubeIndices[10] = 6; m_CubeIndices[11] = 4;	// Triangle 3
	// Top
	m_CubeIndices[12] = 10; m_CubeIndices[13] = 9; m_CubeIndices[14] = 8;	// Triangle 4
	m_CubeIndices[15] = 11; m_CubeIndices[16] = 10; m_CubeIndices[17] = 8;	// Triangle 5
	// Bottom
	m_CubeIndices[18] = 14; m_CubeIndices[19] = 13; m_CubeIndices[20] = 12;	// Triangle 6
	m_CubeIndices[21] = 15; m_CubeIndices[22] = 14; m_CubeIndices[23] = 12;	// Triangle 7
	// Left
	m_CubeIndices[24] = 18; m_CubeIndices[25] = 17; m_CubeIndices[26] = 16;	// Triangle 8
	m_CubeIndices[27] = 19; m_CubeIndices[28] = 18; m_CubeIndices[29] = 16;	// Triangle 9
	// Right
	m_CubeIndices[30] = 22; m_CubeIndices[31] = 21; m_CubeIndices[32] = 20;	// Triangle 10
	m_CubeIndices[33] = 23; m_CubeIndices[34] = 22; m_CubeIndices[35] = 20;	// Triangle 11

	// Create Vertex Buffer
	gD3DDevice->CreateVertexBuffer(
		4 * 6 * sizeof(Vertex),		// Length in bytes to allocate buffer (num quads * num sides * sizeof(Vertex))
		D3DUSAGE_WRITEONLY,		// Usage
		0,						// Used only with FVF, we are not using
		D3DPOOL_MANAGED,		// Memory Pool
		&m_D3DVertexBuffer,	// Vertex Buffer
		0);						// No longer used, set to 0

	// Create Index Buffer
	gD3DDevice->CreateIndexBuffer(
		3 * 12 * sizeof(WORD),	// Length in bytes to allocate buffer (3 verts * num triangles * sizeof(WORD))
		D3DUSAGE_WRITEONLY,		// Usage
		D3DFMT_INDEX16,			// Index Format
		D3DPOOL_MANAGED,		// Memory Pool
		&m_D3DIndexBuffer,		// Index Buffer
		0);						// No longer used
	
	//////////////////////////////////////////////////////////////////////////
	// Accessing vertex buffer memory
	//////////////////////////////////////////////////////////////////////////
	void* pVerts;
	// Lock vertex buffer
	m_D3DVertexBuffer->Lock(
		0,			// Offset to Lock (0 locks entire buffer)
		0,			// Size to Lock (0 locks entire buffer)
		&pVerts,	// Double pointer to data
		0);			// Flags

	// Modify data
	memcpy(pVerts, m_CubeVerts, 4 * 6 * sizeof(Vertex));

	// Unlock vertex buffer
	m_D3DVertexBuffer->Unlock();

	//////////////////////////////////////////////////////////////////////////
	// Accessing index buffer memory
	//////////////////////////////////////////////////////////////////////////
	void* pIndices;
	// Lock index buffer
	m_D3DIndexBuffer->Lock(
		0,			// Offset to Lock (0 locks entire buffer)
		0,			// Size to Lock (0 locks entire buffer)
		&pIndices,	// Double pointer to data
		0);			// Flags

	// Modify data
	memcpy(pIndices, m_CubeIndices, 3 * 12 * sizeof(WORD));

	// Unlock index buffer
	m_D3DIndexBuffer->Unlock();
}

void SkyBox::Render()
{
	gD3DDevice->SetStreamSource(0, m_D3DVertexBuffer, 0, sizeof(Vertex));
	gD3DDevice->SetIndices(m_D3DIndexBuffer);
	gD3DDevice->SetVertexDeclaration(m_D3DVertexDecl);

	D3DXMATRIX worldMat, scaleMat, transMat, WIT;
	D3DXMatrixTranslation(&transMat, 0.0f, 100.0f, 0.0f);
	D3DXMatrixScaling(&scaleMat, 200.0f,100.0f,200.0f);
	worldMat = scaleMat * transMat;
	
	gAssets->GetShader("ProjectB.fx")->SetTechnique(m_hTech);

	//Get light and camera data
	D3DLIGHT9 mainLight = gActors->GetMainLight();
	D3DXMATRIX viewMat = ((Camera*)gActors->GetActiveCamera())->GetView();
	D3DXMATRIX projMat = ((Camera*)gActors->GetActiveCamera())->GetProjection();
	D3DXVECTOR3 camPosition = ((Camera*)gActors->GetActiveCamera())->GetPosition();

	gAssets->GetShader("ProjectB.fx")->SetFloatArray("eyePos", (float*)camPosition, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightPos", (float*)&mainLight.Position, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("ambientLight", (float*)&mainLight.Ambient, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("specularLight", (float*)&mainLight.Specular, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("diffuseLight", (float*)&mainLight.Diffuse, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightAttenuation", (float*)&D3DXVECTOR3(mainLight.Attenuation0, mainLight.Attenuation1, mainLight.Attenuation2), 3);

	D3DXMatrixInverse(&WIT, 0, &worldMat);
	D3DXMatrixTranspose(&WIT, &WIT);
	gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
	gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
	gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);
	gAssets->GetShader("ProjectB.fx")->SetValue("ambientMaterial", &m_Material.Ambient, sizeof(D3DXCOLOR));
	gAssets->GetShader("ProjectB.fx")->SetValue("diffuseMaterial", &m_Material.Diffuse, sizeof(D3DXCOLOR));
	gAssets->GetShader("ProjectB.fx")->SetValue("specularMaterial", &m_Material.Specular, sizeof(D3DXCOLOR));
	gAssets->GetShader("ProjectB.fx")->SetFloat("specularPower", m_Material.Power);
	gAssets->GetShader("ProjectB.fx")->SetTexture("tex", gAssets->GetTexture("SkyBox.png"));

	// Begin Effect
	UINT numPasses = 0;
	gAssets->GetShader("ProjectB.fx")->Begin(&numPasses, 0);

	for(UINT i = 0; i < numPasses; ++i)
	{
		gAssets->GetShader("ProjectB.fx")->BeginPass(i);
		gD3DDevice->DrawIndexedPrimitive(D3DPT_TRIANGLELIST, 0, 0, 4 * 6, 0, 12);
		gAssets->GetShader("ProjectB.fx")->EndPass();
	}
	gAssets->GetShader("ProjectB.fx")->End();
}