#include "Steering.h"
#include "Defines.h"
#include "DirectX9App.h"
#include "AITank.h"
#include "ActorManager.h"


Steering::Steering(Actor* owner):m_Owner(owner)
{
	m_Seeking = false;
	m_Arriving = false;
	m_Fleeing = false;
}

Steering::~Steering()
{

}

void Steering::Seek(D3DXVECTOR3 target, float weight)
{
	m_Seeking = true;
	m_SeekTarget = target;
	m_SeekWeight = weight;
}

void Steering::SeekOff()
{
	m_Seeking = false;
}

bool Steering::IsSeeking()
{
	return m_Seeking;
}

void Steering::Flee(D3DXVECTOR3 target, float weight)
{
	m_Fleeing = true;
	m_FleeTarget = target;
	m_FleeWeight = weight;
}

void Steering::FleeOff()
{
	m_Fleeing = false;
}

bool Steering::IsFleeing()
{
	return m_Fleeing;
}

void Steering::Arrive(D3DXVECTOR3 target, float arriveRadius, float arriveSlowRadius, float arriveTime, float weight)
{
	m_Arriving = true;
	m_ArriveTarget = target;
	m_ArriveRadius = arriveRadius;
	m_ArriveSlowRadius = arriveSlowRadius;
	m_ArriveTime = arriveTime;
	m_ArriveWeight = weight;
}

void Steering::ArriveOff()
{
	m_Arriving = false;
}

bool Steering::IsArriving()
{
	return m_Arriving;
}

SteeringForce Steering::Calculate()
{
	SteeringForce output;

	//Init Steering Force
	output.linear = D3DXVECTOR3(0.0f, 0.0f, 0.0f);
	
	//Temp variables for calculation
	D3DXVECTOR3 toTarget;
	//Calculate Seek Steering Force
	if(m_Seeking)
	{
		toTarget = m_SeekTarget - ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetPosition();
		D3DXVec3Normalize(&toTarget, &toTarget);
		toTarget *= ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetMaxAccel();
		output.linear += toTarget * m_SeekWeight;
	}

	//Calculate Flee Steering Force
	if(m_Fleeing)
	{
		toTarget = ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetPosition() - m_FleeTarget;
		D3DXVec3Normalize(&toTarget, &toTarget);
		toTarget *= ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetMaxAccel();
		output.linear += toTarget * m_FleeWeight;
	}

	//Calculate Arrive Steering Force
	if(m_Arriving)
	{
		toTarget = m_ArriveTarget - ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetPosition();
		float distanceToTarget = D3DXVec3Length(&toTarget);
			
		//If we're outside the target radius, calculate the steering force
		if(distanceToTarget > m_ArriveRadius)
		{
			float targetSpeed;
			//Full speed if we're not at the slow range yet.
			if(distanceToTarget > m_ArriveSlowRadius)
			{
				targetSpeed = ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetMaxSpeed();
			}
			//Otherwise, scale based on distance
			else
			{
				targetSpeed = ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetMaxSpeed() * distanceToTarget / m_ArriveSlowRadius;
			}	
			//Generate the target velocity
			D3DXVec3Normalize(&toTarget, &toTarget);	//Velocity Direction
			toTarget *= targetSpeed;					//Velocity Magnitude
			
			//Translate this into target acceleration
			toTarget -= ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetVelocity();
			toTarget *= ( 1.0f / m_ArriveTime );

			output.linear += toTarget * m_ArriveWeight;
		}
		
	}
			
	//Clamp output to max accel
	D3DXVec3Normalize(&output.linear, &output.linear);
	output.linear *= ((AITank*)gActors->GetActiveAITank())->GetTank()->GetPhysics()->GetMaxAccel();
		
	//Output steering force
	return output;
}