#include "Tank.h"
#include "DirectX9App.h"
#include "D3DXMathFunctions.h"
#include <sstream>
#include "ActorManager.h"
#include "Camera.h"
#include "ActorFactory.h"
#include "AssetManager.h"
#include "FMODAudio.h"


Tank::Tank(int owner):m_Owner(owner)
{
	ZeroMemory(&m_Material, sizeof(m_Material));

	m_Material.Ambient = D3DXCOLOR(0.2f, 0.2f, 0.2f, 1.0f);		// Ambient color reflected
	m_Material.Diffuse = D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f);		// Diffuse color reflected
	m_Material.Emissive = D3DXCOLOR(0.0f, 0.0f, 0.0f, 1.0f);		// Emissive color reflected
	m_Material.Specular = D3DXCOLOR(0.8f, 0.8f, 0.8f, 0.8f);		// Specular
	m_Material.Power = 50000.0f;									// Specular highlight intensity
	
	m_hTech = gAssets->GetShader("ProjectB.fx")->GetTechniqueByName("tech0");

	m_BaseDirection = D3DXVECTOR3(0.0f, 0.0f, -1.0f);
	m_TurretDirection = D3DXVECTOR3(0.0f, 0.0f, 1.0f);
	m_MachineGunTimer = 0.0f;
	m_MainGunTimer = 0.0f;
	m_MainGunAmmo = 5;
	m_MachineGunAmmo = 50;
}

void Tank::RotateTurret(float degree)
{
	m_TurretRotation += degree;
	D3DXMATRIX rotation;
	D3DXMatrixRotationY(&rotation, degree);
	m_TurretDirection = TransformVector(&m_TurretDirection, &rotation);
	D3DXVec3Normalize(&m_TurretDirection, &m_TurretDirection);
	
}

void Tank::RotateBase(float degree)
{
	m_BaseRotation += degree;
	D3DXMATRIX rotation;
	D3DXMatrixRotationY(&rotation, degree);
	m_BaseDirection = TransformVector(&m_BaseDirection, &rotation);
	D3DXVec3Normalize(&m_BaseDirection, &m_BaseDirection);

}

void Tank::PitchCannon(float degree)
{
	m_CannonPitch += degree;

	if(m_CannonPitch > m_MaxCannonPitch)
	{
		m_CannonPitch = m_MaxCannonPitch;
	}
	if(m_CannonPitch < -m_MaxCannonPitch)
	{
		m_CannonPitch = -m_CannonPitch;
	}
}

void Tank::Update(float dt)
{
	m_MachineGunTimer -= dt;
	m_MainGunTimer -= dt;
	m_Physics.Update(dt);
		
	//Clamp to ground
	if(m_Physics.GetPosition().y <= 0.0f)
		m_Physics.SetPosition(D3DXVECTOR3(m_Physics.GetPosition().x, 0.0f, m_Physics.GetPosition().z));

	//Clamp to game level
	if(m_Physics.GetPosition().x > 180.0f)
		m_Physics.SetPosition(D3DXVECTOR3(180.0f, m_Physics.GetPosition().y, m_Physics.GetPosition().z));
	if(m_Physics.GetPosition().x <= -180.0f)
		m_Physics.SetPosition(D3DXVECTOR3(-180.0f, m_Physics.GetPosition().y, m_Physics.GetPosition().z));
	if(m_Physics.GetPosition().z > 180.0f)
		m_Physics.SetPosition(D3DXVECTOR3(m_Physics.GetPosition().x, m_Physics.GetPosition().y, 180.0f));
	if(m_Physics.GetPosition().z <= -180.0f)
		m_Physics.SetPosition(D3DXVECTOR3(m_Physics.GetPosition().x, m_Physics.GetPosition().y, -180.0f));
}

void Tank::Render()
{
	
	D3DXMATRIX transMat, rotMat, scaleMat, worldMat, WIT;
	D3DXMatrixIdentity(&transMat);
	D3DXMatrixIdentity(&scaleMat);
	D3DXMatrixIdentity(&rotMat);
	D3DXMatrixIdentity(&worldMat);
	
	gAssets->GetShader("ProjectB.fx")->SetTechnique(m_hTech);

	//Get light and camera data
	D3DLIGHT9 mainLight = gActors->GetMainLight();
	D3DXMATRIX viewMat = ((Camera*)gActors->GetActiveCamera())->GetView();
	D3DXMATRIX projMat = ((Camera*)gActors->GetActiveCamera())->GetProjection();
	D3DXVECTOR3 camPosition = ((Camera*)gActors->GetActiveCamera())->GetPosition();

	gAssets->GetShader("ProjectB.fx")->SetFloatArray("eyePos", (float*)camPosition, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightPos", (float*)&mainLight.Position, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("ambientLight", (float*)&mainLight.Ambient, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("specularLight", (float*)&mainLight.Specular, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("diffuseLight", (float*)&mainLight.Diffuse, 3);
	gAssets->GetShader("ProjectB.fx")->SetFloatArray("lightAttenuation", (float*)&D3DXVECTOR3(mainLight.Attenuation0, mainLight.Attenuation1, mainLight.Attenuation2), 3);

	
	// Begin Effect
	UINT numPasses = 0;
	gAssets->GetShader("ProjectB.fx")->Begin(&numPasses, 0);

	for(UINT i = 0; i < numPasses; ++i)
	{
		gAssets->GetShader("ProjectB.fx")->BeginPass(i);
	
	
	//Tank Base
		
	// Calculate Matrix Transform
		D3DXMatrixScaling(&scaleMat, 1.0f, 1.0f, 1.0f);			// Scaling
		D3DXMatrixRotationYawPitchRoll(&rotMat, m_BaseRotation, 0.0f, 0.0f); // Rotation on Yaw, Pitch, and Roll
		D3DXMatrixTranslation(&transMat, m_Physics.GetPosition().x, m_Physics.GetPosition().y, m_Physics.GetPosition().z);		// Translation
		D3DXMatrixMultiply(&scaleMat, &scaleMat, &rotMat);		// Multiply scale and rotation, store in scale
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);	// Multiply scale and translation, store in world
		
		D3DXMatrixInverse(&WIT, 0, &worldMat);
		D3DXMatrixTranspose(&WIT, &WIT);
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);
		gAssets->GetShader("ProjectB.fx")->SetValue("ambientMaterial", &m_Material.Ambient, sizeof(D3DXCOLOR));
		gAssets->GetShader("ProjectB.fx")->SetValue("diffuseMaterial", &m_Material.Diffuse, sizeof(D3DXCOLOR));
		gAssets->GetShader("ProjectB.fx")->SetValue("specularMaterial", &m_Material.Specular, sizeof(D3DXCOLOR));
		gAssets->GetShader("ProjectB.fx")->SetFloat("specularPower", m_Material.Power);
		gAssets->GetShader("ProjectB.fx")->SetTexture("tex", gAssets->GetTexture("GreyTank.png"));

		gAssets->GetShader("ProjectB.fx")->CommitChanges();
		
		gAssets->GetMesh("TankBase.x")->DrawSubset(0);

		//Tank Turret

		// Calculate Matrix Transform
		D3DXMatrixScaling(&scaleMat, 1.0f, 1.0f, 1.0f);			// Scaling
		D3DXMatrixTranslation(&transMat, m_Physics.GetPosition().x, m_Physics.GetPosition().y, m_Physics.GetPosition().z);
		D3DXMatrixRotationYawPitchRoll(&rotMat, m_TurretRotation, 0.0f, 0.0f); // Rotation on Yaw, Pitch, and Roll
		D3DXMatrixMultiply(&scaleMat, &scaleMat, &rotMat);		// Multiply scale and rotation, store in scale
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);	// Multiply scale and translation, store in world

		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);

		gAssets->GetShader("ProjectB.fx")->CommitChanges();
		
		gAssets->GetMesh("TankTurret.x")->DrawSubset(0);

		//Tank Cannon

		// Calculate Matrix Transform
		D3DXMatrixScaling(&scaleMat, 1.0f, 1.0f, 1.0f);			// Scaling
		D3DXMatrixTranslation(&transMat, m_Physics.GetPosition().x, m_Physics.GetPosition().y, m_Physics.GetPosition().z);
		D3DXMatrixRotationYawPitchRoll(&rotMat, m_TurretRotation, -0.15f, 0.0f); // Rotation on Yaw, Pitch, and Roll
		D3DXMatrixMultiply(&scaleMat, &scaleMat, &rotMat);		// Multiply scale and rotation, store in scale
		D3DXMatrixMultiply(&worldMat, &scaleMat, &transMat);	// Multiply scale and translation, store in world

		
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldViewProjMat", &(worldMat * viewMat * projMat));
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldInverseTransposeMat", &WIT);
		gAssets->GetShader("ProjectB.fx")->SetMatrix("worldMat", &worldMat);

		gAssets->GetShader("ProjectB.fx")->CommitChanges();
		

		gAssets->GetMesh("TankBarrel.x")->DrawSubset(0);

		
		gAssets->GetShader("ProjectB.fx")->EndPass();
	}

	gAssets->GetShader("ProjectB.fx")->End();
	
}

void Tank::FireMachineGuns()
{
	if(m_MachineGunTimer <= 0.0f)
	{
		if(m_MachineGunAmmo > 0)
		{
			gFactory->CreateBullet(m_Physics.GetPosition() + D3DXVECTOR3(0.0f, 2.0f, 0.0f) + m_TurretDirection * 3, m_TurretDirection, m_Owner);
			gFactory->CreateBullet(m_Physics.GetPosition() + D3DXVECTOR3(0.0f, 2.0f, 0.0f) + m_TurretDirection * 3, m_TurretDirection, m_Owner);
			m_MachineGunTimer = 0.125f;
			m_MachineGunAmmo -= 2;
			if(m_Owner == gActors->GetPlayer()->GetID())
			gAudio->PlaySoundFX("Machinegun_Fire.wav");
		}
	}
}

void Tank::FireMainGun()
{
	if(m_MainGunTimer <= 0.0f)
	{
		if(m_MainGunAmmo > 0)
		{
			gFactory->CreateShell(m_Physics.GetPosition() + D3DXVECTOR3(0.0f, 2.0f, 0.0f) + m_TurretDirection * 3, m_TurretDirection, m_Owner);
			m_MainGunTimer = 3.0f;
			m_MainGunAmmo--;
			if(m_Owner == gActors->GetPlayer()->GetID())
			gAudio->PlaySoundFX("Cannon_Fire.wav");
		}
	}
}

void Tank::ReloadMachineGuns(int amount)
{
	m_MachineGunAmmo + amount > m_MachineGunMaxAmmo ? m_MachineGunAmmo = m_MachineGunMaxAmmo : m_MachineGunAmmo += amount ;
}

void Tank::ReloadMainGun(int amount)
{
	m_MainGunAmmo + amount > m_MainGunMaxAmmo ? m_MainGunAmmo = m_MainGunMaxAmmo : m_MainGunAmmo += amount ;
}

const int Tank::m_MainGunMaxAmmo = 10;
const int Tank::m_MachineGunMaxAmmo = 100;
const float Tank::m_MaxCannonPitch = 10.0f;

