// Tank.h - Actual player or AI controlled tank object

#pragma once

#include "Physics.h"
#include "d3dx9.h"

class Tank
{
	Physics m_Physics;
	int m_Owner;

	D3DXHANDLE				m_hTech;
	D3DMATERIAL9			m_Material;
	
	//Rotation data
	float					m_BaseRotation;
	float					m_TurretRotation;
	float					m_CannonPitch;
	static const float		m_MaxCannonPitch;
	D3DXVECTOR3				m_BaseDirection;
	D3DXVECTOR3				m_TurretDirection;

	//Ammo data
	int						m_MainGunAmmo;
	int						m_MachineGunAmmo;
	static const int		m_MainGunMaxAmmo;
	static const int		m_MachineGunMaxAmmo;
	float					m_MainGunTimer;
	float					m_MachineGunTimer;
		
public:
	Tank(int owner);

	//Rotation
	void RotateTurret(float degree);
	void RotateBase(float degree);
	void PitchCannon(float degree);
	D3DXVECTOR3 GetBaseDirection(){return m_BaseDirection;}
	float GetTurretRotation(){return m_TurretRotation;}
	int GetMainGunAmmo(){return m_MainGunAmmo;}
	int GetMachineGunAmmo(){return m_MachineGunAmmo;}
	int GetOwner(){return m_Owner;}
	
	//Fire Control
	void FireMainGun();
	void FireMachineGuns();
	void ReloadMainGun(int amount);
	void ReloadMachineGuns(int amount);

	//Main Control
	void Update(float dt);
	void Render();

	//Accessors
	Physics* GetPhysics(){return &m_Physics;}
};
